/********************************************************************************* 
 *     COPYRIGHT NOTICE:
 *     Copyright © 2013 Andrew Smith (GNA SERVICES INC) <Andrew@GNAServicesInc.com>
 *     All Rights Reserved.
 *
 *   This file, tpad_config.c , is part of tpad.
 *
 *   tpad is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *   tpad is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tpad.  If not, see <http://www.gnu.org/licenses/>.
 ********************************************************************************/
#include "tpad_headers.h"
//////////////////////////////////////////////////////////////////////////
/***********************************DEC**********************************/
//////////////////////////////////////////////////////////////////////////

pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
static cfgSet userCfg;
static char* ccfile;
//////////////////////////////////////////////////////////////////////////
void cfg_read_in();
void set_conf_defaults();
void clean_userCfg();
inline int is_userCfg_valid();
inline void print_userCfg(char *fun);
void config_file();
int str_size(char* string);
int write_out();
void reset_config ();
void read_in();
void *do_write_out();
//////////////////////////////////////////////////////////////////////////
/***********************************PUB**********************************/
//////////////////////////////////////////////////////////////////////////
void cfg_on_exit(){
}
int cfg_show_full_path(){
	return(userCfg.show_full_path);
}
int cfg_lang(){
	return(userCfg.show_lang);
}
int cfg_line(){
	return(userCfg.show_line_numbers);
}
int cfg_spell(){
	return(userCfg.show_spelling_errors);
}
int cfg_sWidth(){
	return(userCfg.screenWidth);
	//return(cfg_wWidth()); //Place Holder. 
}
int cfg_sHeight(){
	return(userCfg.screenHeight);
	//return(cfg_wHeight()); //Place Holder. 
}
int cfg_wWidth(){
	return(userCfg.defualt_window_width);
}
int cfg_wHeight(){
	return(userCfg.default_window_height);
}
int cfg_undo(){
	return(userCfg.undo_level);
}
int cfg_line_wrap(){
	return(userCfg.line_wrap);
}

#ifdef AUTO_TAB_TOGGLE
int cfg_auto_tab(){
	return(userCfg.auto_tab);
}
#endif
///////////////////////////////////////////////////////
void cfg_set_show_line(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_line_numbers=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_line",0,0);
}
void cfg_set_show_lang(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_lang=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_lang",0,0);
}
void cfg_set_show_full_path(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_full_path=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_show_full_path",0,0);
}
void cfg_set_show_spelling(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_spelling_errors=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_spelling",0,0);
}
void cfg_set_screen_width(int opt){
	#ifdef DEBUG_TOGGLE
	g_print("\nSet Screen Width\t=\t%i\n",opt);
	#endif
	if(opt > 0) userCfg.screenWidth=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_screen_width",0,0);
}
void cfg_set_screen_height(int opt){
	#ifdef DEBUG_TOGGLE
	g_print("\nSet Screen Height\t=\t%i\n",opt);
	#endif
	if(opt > 0) userCfg.screenHeight=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_screen_height",0,0);
}
void cfg_set_default_width(int opt){
	if(opt >= 25) userCfg.defualt_window_width=opt;
	else gerror_warn(_ERROR_TOO_SMALL,"cfg_set_default_width",0,0);
}
void cfg_set_default_height(int opt){
	if(opt >= 50 || opt > 2147483647) userCfg.default_window_height=opt;
	else gerror_warn(_ERROR_TOO_SMALL,"cfg_set_default_height",0,0);
}
void cfg_set_undo(int opt){
	if(opt >= 0 || opt > 2147483647) userCfg.undo_level=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_undo",0,0);
}
void cfg_set_show_line_wrap(int opt){
	if(opt >= 0 && opt <=1) userCfg.line_wrap=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_line_wrap",0,0);
}
#ifdef AUTO_TAB_TOGGLE
void cfg_set_auto_tab(int opt){
	if(opt >= 0 && opt <=1) userCfg.auto_tab=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_auto_tab",0,0);
}
#endif
//////////////////////////////////////////////////////////////////////////

void config_setup(){
	config_file();
	set_conf_defaults();
	if(access(ccfile, R_OK|W_OK ) != -1) cfg_read_in();
	else cfg_save();
}
//////////////////////////////////////////////////////////////////////////
void cfg_save(){
	if (!is_userCfg_valid()){
		print_userCfg("cfg_save()");
		clean_userCfg();
	}
	write_out();
}

//////////////////////////////////////////////////////////////////////////
int str_size(char* string){
 mbstate_t t;
 char *scopy = string;
 memset (&t, '\0', sizeof (t));
 return(mbsrtowcs (NULL,(const char **) &scopy, strlen (scopy), &t));
}

//////////////////////////////////////////////////////////////////////////
/***********************************PRI**********************************/
//////////////////////////////////////////////////////////////////////////


void cfg_read_in(){
	read_in();
	if (!is_userCfg_valid()){
		print_userCfg("cfg_read_in()");
		clean_userCfg();
	}
}
//////////////////////////////////////////////////////////////////////////
void set_conf_defaults(){
	userCfg.show_line_numbers=1;
	userCfg.show_lang=0;
	userCfg.show_spelling_errors=0;
	userCfg.defualt_window_width=800;
	userCfg.default_window_height=900;
	userCfg.undo_level=0;
	userCfg.line_wrap=1;
	userCfg.show_full_path=1;
	userCfg.screenWidth=0;
	userCfg.screenHeight=0;
	#ifdef AUTO_TAB_TOGGLE
	userCfg.auto_tab=0;
	#endif
}
//////////////////////////////////////////////////////////////////////////
inline void print_userCfg(char *fun){
	#ifdef DEBUG_TOGGLE
	print_title("WARNING - ERROR!");
	printf("\nINCLUDE THE FOLLOWING IN YOUR BUG REPORT:\n");
	print_title("BEGIN DEBUGGING INFO");
	printf("TPAD VERSION NUMBER:\t%s\n",VERSION);
	printf("PATH TO CONFIG FILE:\t%s\n",ccfile);
	printf("OUTOF RANGE / INVALID DATA IN: cfgSet userCfg\n");
	printf("IN FUNCTION:\t%s\n",fun);
	printf("%s =\t%i\n","userCfg.show_line_numbers",userCfg.show_line_numbers);
	printf("%s =\t%i\n","userCfg.show_lang",userCfg.show_lang);
	printf("%s =\t%i\n","userCfg.show_spelling_errors",userCfg.show_spelling_errors);
	printf("%s =\t%i\n","userCfg.defualt_window_width",userCfg.defualt_window_width);
	printf("%s =\t%i\n","userCfg.default_window_height",userCfg.default_window_height);
	printf("%s =\t%i\n","userCfg.undo_level",userCfg.undo_level);
	printf("%s =\t%i\n","userCfg.line_wrap",userCfg.line_wrap);
	printf("%s =\t%i\n","userCfg.show_full_path",userCfg.show_full_path);
	#ifdef AUTO_TAB_TOGGLE
	printf("%s =\t%i\n","userCfg.auto_tab",userCfg.auto_tab);
	#endif
	print_title("END DEBUGGING INFO");	
	#endif
}
//////////////////////////////////////////////////////////////////////////
int is_userCfg_valid(){
        if(userCfg.show_line_numbers < 0 || userCfg.show_line_numbers > 1) goto caseBad;
		if(userCfg.show_lang < 0 || userCfg.show_lang > 1) goto caseBad;
		if(userCfg.show_spelling_errors < 0 || userCfg.show_spelling_errors > 1) goto caseBad;
		if(userCfg.defualt_window_width < 10 || userCfg.defualt_window_width > WIDTH_MAX) goto caseBad;
		if(userCfg.default_window_height < 10 || userCfg.default_window_height > HEIGHT_MAX) goto caseBad;
		if(userCfg.undo_level < 0 || userCfg.undo_level > UNDOMAX) goto caseBad;
		if(userCfg.line_wrap < 0 || userCfg.line_wrap > 1) goto caseBad;
		if(userCfg.show_full_path < 0 || userCfg.show_full_path > 1) goto caseBad;
		#ifdef AUTO_TAB_TOGGLE
		if(userCfg.auto_tab < 0 || userCfg.auto_tab > 1) goto caseBad;
		#endif
		return(1);
	caseBad:
		return(0);
}
//////////////////////////////////////////////////////////////////////////
void clean_userCfg(){
        if(userCfg.show_line_numbers < 0 || userCfg.show_line_numbers > 1) userCfg.show_line_numbers=1;
		if(userCfg.show_lang < 0 || userCfg.show_lang > 1) userCfg.show_lang=0;
		if(userCfg.show_spelling_errors < 0 || userCfg.show_spelling_errors > 1) userCfg.show_spelling_errors=0;
		if(userCfg.screenWidth > 0 ){//&& userCfg.screenWidth <= WIDTH_MAX){
			//if(userCfg.defualt_window_width < 10 ||
			if(userCfg.defualt_window_width > userCfg.screenWidth) userCfg.defualt_window_width=userCfg.screenWidth;
		}
		else{
			if(userCfg.defualt_window_width < 10 || userCfg.defualt_window_width > WIDTH_MAX) userCfg.defualt_window_width=_DEFAULT_WINDOW_WIDTH;
		}
		if (userCfg.screenHeight > 0 ){// && userCfg.screenHeight <= HEIGHT_MAX) {
			if(userCfg.default_window_height > userCfg.screenHeight) userCfg.default_window_height=userCfg.screenHeight;
		}
		else{
			if(userCfg.default_window_height < 10 || userCfg.default_window_height > HEIGHT_MAX) userCfg.default_window_height=_DEFAULT_WINDOW_HEIGHT;
		}
		
		if(userCfg.undo_level < 0 || userCfg.undo_level > UNDOMAX) userCfg.undo_level=0;
		if(userCfg.line_wrap < 0 || userCfg.line_wrap > 1) userCfg.line_wrap=1;
		if(userCfg.show_full_path < 0 || userCfg.show_full_path > 1) userCfg.show_full_path=1;
		#ifdef AUTO_TAB_TOGGLE
		if(userCfg.auto_tab < 0 || userCfg.auto_tab > 1) userCfg.auto_tab=0;
		#endif
}
//////////////////////////////////////////////////////////////////////////
void config_file(){
	struct passwd *pw = getpwuid(syscall(__NR_getuid));
	ccfile = g_strconcat((gchar *)pw->pw_dir,(gchar *)CONFIG_FILE_SUFFIX,NULL);
}
//////////////////////////////////////////////////////////////////////////
int write_out(){
	int rc;
	pthread_t thread;
	if( (rc=pthread_create( &thread, NULL, do_write_out, NULL)) ){
    	return(1);
	}
	else {
	 pthread_join( thread, NULL);
	 return(0);
	}
}
//////////////////////////////////////////////////////////////////////////
void reset_config (){
	 if (remove(ccfile) != 0 ){
		gerror_warn(_BAD_CFG_REPLACE_FAIL,ccfile,1,1); 
	 }
	 else gerror_warn(_CFG_FILE_INVALID,_REMOVED_INVALID_CFG_FILE,1,1);
		 
}
//////////////////////////////////////////////////////////////////////////
void read_in()
{
	FILE *ptr_cfg;
	ptr_cfg=fopen(ccfile,"rb");
	if (!ptr_cfg) {
		reset_config ();
	}
	else {
		//Success opening, read data into the struct 
		fseek(ptr_cfg, sizeof(cfgSet), SEEK_END);
		rewind(ptr_cfg);
		size_t result;
		long lSize =1;
		result=fread(&userCfg,sizeof(cfgSet),1,ptr_cfg);
		if (result != lSize){
			reset_config ();
		}
		fclose(ptr_cfg);
	}
}
//////////////////////////////////////////////////////////////////////////	
void *do_write_out()
{
	FILE *ptr_cfg;
	pthread_mutex_lock( &mutex );
	
	ptr_cfg=fopen(ccfile,"wb");
	
	if(!ptr_cfg) reset_config ();
	else {
		fwrite(&userCfg, sizeof(cfgSet),1,ptr_cfg);
		fclose(ptr_cfg);
		}
	pthread_mutex_unlock( &mutex );
	return(0);
}

//////////////////////////////////////////////////////////////////////////