/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/********************************************************************************* 
 *     COPYRIGHT NOTICE:
 *     Copyright © 2013 Andrew Smith (GNA SERVICES INC) <Andrew@GNAServicesInc.com>
 *     All Rights Reserved.
 *
 *   This file, tpad_file.c , is part of tpad.
 *
 *   tpad is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *   tpad is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tpad.  If not, see <http://www.gnu.org/licenses/>.
 ********************************************************************************/

#include "tpad_headers.h"

static gchar *cRpath=NULL;

gchar *clean_path(gchar *path){
	GError *error = NULL;
	gchar *buf_word=NULL;
	gchar *bwords=NULL;
	bwords=path;
	gint bwordsize=str_size(bwords);
	if (bwordsize >= (gint)1)
		{
		buf_word=g_strdup(g_convert(bwords, (gint) -1, (gchar*)"UTF-8", g_get_codeset(),NULL, NULL, &error));	
		if (error != NULL) 
			{
			print(g_strdup(error->message));
			return(NULL);
			}
			else return(g_strdup(buf_word));	
			}
	return(NULL);
}

gchar* link_resolve(gchar* file)
{
	gint path_max=0;
           #ifdef PATH_MAX
             path_max = PATH_MAX;
           #else
             path_max = pathconf(path, _PC_PATH_MAX);
             if (path_max <= 0) path_max = 4096;
           #endif
		 gchar* c_rpath=NULL;
		c_rpath=realpath(g_strdup(file),NULL);
	if(c_rpath == NULL)
	{
		static gchar *OutputError=NULL;
		if (errno){	
		OutputError=g_strconcat("Error resolving file path for:\n",
		                        file,"Reason:\n",
		                        g_strdup(strerror(errno)),"\n", NULL);
		}
		else{
		OutputError=g_strconcat("Error resolving file path for:\n",
		                        file,"Reason:\nUNK ERROR\n",NULL);
		}
		gerror_warn(OutputError,
		            "file_system.c->link_resolve->realpath: c_rpath == NULL",
		            TRUE,FALSE);
		return(NULL);
	}
	else return(g_strdup(c_rpath));	
}

gint is_path_a_directory(gchar *path){
	if (get_file_type(g_strdup(clean_path( g_strdup(path)))) == _FILE_IS_A_DIRECTORY) return(TRUE);
	else return(FALSE);

}

gint get_file_type( gchar *file){
	struct stat buf;
	if (stat(file, &buf)== -1) {
			 print(g_strconcat("ERROR  Getting Information on file:  ",file,"\n",NULL));
			return( _ERROR_FILE_GET_TYPE);
          }  
	else{
			if (S_ISLNK(buf.st_mode)) return(_FILE_IS_A_LINK);
		    else if(S_ISDIR(buf.st_mode)) return(_FILE_IS_A_DIRECTORY);
			else if(S_ISREG(buf.st_mode)) return( _FILE_IS_A_REGRULAR_FILE);
			else return( _ERROR_FILE_GET_TYPE);
	}
}

gchar* getcRpath(void){

	if(cRpath!=NULL) return(g_strdup(cRpath));
	else return((gchar*)NULL);
}
 gchar* check_file(gchar *afile)
{
 static gchar* file=NULL;
 static gchar* nfile=NULL;

	file= g_strdup(clean_path( g_strdup(afile)));


switch(get_file_type(file))
	{
	case _FILE_IS_A_LINK:

					nfile= link_resolve( g_strdup(file));
					if(get_file_type( g_strdup(nfile)) ==  _FILE_IS_A_REGRULAR_FILE) {
						return(g_strdup(nfile));
					}
			       else  return((gchar*) NULL);
			break;
	case _FILE_IS_A_DIRECTORY:
			// Holding spot for now
			cRpath=g_strdup(file);
			return((gchar*) NULL);
			break;
	case  _FILE_IS_A_REGRULAR_FILE:
				return(g_strdup(file));
			break;
	case  _ERROR_FILE_GET_TYPE:
						// Holding spot for now
			return((gchar*) NULL);
			break;
	default:
							// Holding spot for now
			return((gchar*) NULL);
			break;

	}
	
}